@@ -1,19 +1,39 @@
+"""
+---
+title: Evaluate GPT-NeoX using LLM.int8() quantization on test suite
+summary: >
+     Evaluate GPT-NeoX using LLM.int8() quantization on test suite
+---
+
+#  Evaluate GPT-NeoX using LLM.int8() quantization on test suite
+
+This code evaluate [GPT-NeoX](../index.html) using, on a suite of tasks.
+"""
+
 import torch
 from torch import nn
 
-from labml import monit
 from labml_nn.neox.evaluation import run_eval_harness
 from labml_nn.neox.model import LayerGenerator
 
-if __name__ == '__main__':
+
+def main():
+    # Device
     device = torch.device('cuda:0')
+    # Load layers
     layers = list(LayerGenerator(is_clone_layers=True,
                                  filter_layers=None,
                                  dtype=torch.float16,
                                  device=device
                                  ).load())
 
-    with monit.section('Sequential'):
-        model = nn.Sequential(*layers)
+    # Create `nn.Sequential` model
+    model = nn.Sequential(*layers)
 
+    # Run [evaluation harness](index.html)
     print(run_eval_harness(model, 'half_precision', ['lambada'], device))
+
+
+#
+if __name__ == '__main__':
+    main()
