@@ -15,15 +15,15 @@ This is a [PyTorch](https://pytorch.org) implementation of the paper
  [Generalization through Memorization: Nearest Neighbor Language Models](https://papers.labml.ai/paper/1911.00172).
 It uses k-nearest neighbors to  improve perplexity of autoregressive transformer models.
 
-An autoregressive language model estimates $p(w_t | \color{yellowgreen}{c_t})$,
+An autoregressive language model estimates $p(w_t | \textcolor{yellowgreen}{c_t})$,
  where $w_t$ is the token at step $t$
- and $c_t$ is the context, $\color{yellowgreen}{c_t} = (w_1, w_2, ..., w_{t-1})$.
+ and $c_t$ is the context, $\textcolor{yellowgreen}{c_t} = (w_1, w_2, ..., w_{t-1})$.
 
-This paper, improves  $p(w_t | \color{yellowgreen}{c_t})$ using a k-nearest neighbor search
- on key-value pairs $\big(f(c_i), w_i\big)$, with search key $f(\color{yellowgreen}{c_t})$.
- Here $f(\color{yellowgreen}{c_t})$ is an embedding of the context $\color{yellowgreen}{c_t}$.
+This paper, improves  $p(w_t | \textcolor{yellowgreen}{c_t})$ using a k-nearest neighbor search
+ on key-value pairs $\big(f(c_i), w_i\big)$, with search key $f(\textcolor{yellowgreen}{c_t})$.
+ Here $f(\textcolor{yellowgreen}{c_t})$ is an embedding of the context $\textcolor{yellowgreen}{c_t}$.
  The paper (and this implementation) uses the **input to the feed-forward layer of the
- final layer of the transformer** as $f(\color{yellowgreen}{c_t})$.
+ final layer of the transformer** as $f(\textcolor{yellowgreen}{c_t})$.
 
 We use [FAISS](https://github.com/facebookresearch/faiss) to index $f(c_i)$.
 
@@ -34,7 +34,7 @@ So to run $k$NN-LM we need to:
 * [Train a transformer model](train_model.html)
 * [Build an index](build_index.html) of $\big(f(c_i), w_i\big)$
 * [Evaluate kNN-ML](eval_knn.html) using $k$NN seach on $\big(f(c_i), w_i\big)$
-with  $f(\color{yellowgreen}{c_t})$
+with  $f(\textcolor{yellowgreen}{c_t})$
 
 This experiment uses a small dataset so that we can run this without using up a few hundred giga-bytes
 of disk space for the index.
